<?php

namespace Ivoz\Provider\Domain\Service\Friend;

use Ivoz\Core\Domain\Assert\Assertion;
use Ivoz\Core\Domain\Service\DomainEventSubscriberInterface;
use Ivoz\Core\Domain\Service\LifecycleEventHandlerInterface;
use Ivoz\Core\Domain\Service\LifecycleServiceCollectionInterface;
use Ivoz\Core\Domain\Service\LifecycleServiceCollectionTrait;

/**
 * @codeCoverageIgnore
 */
class FriendLifecycleServiceCollection implements LifecycleServiceCollectionInterface
{
    use LifecycleServiceCollectionTrait;

    /** @var array<array-key, array> $bindedBaseServices */
    public static $bindedBaseServices = [
        "pre_persist" =>
        [
            \Ivoz\Provider\Domain\Service\Friend\AvoidUpdateCompany::class => 200,
            \Ivoz\Provider\Domain\Service\Friend\CheckUniqueness::class => 200,
            \Ivoz\Provider\Domain\Service\Friend\SanitizeInterVpbxPriority::class => 200,
        ],
        "post_persist" =>
        [
            \Ivoz\Ast\Domain\Service\PsEndpoint\UpdateByFriend::class => 10,
            \Ivoz\Ast\Domain\Service\PsIdentify\UpdateByFriend::class => 200,
        ],
        "post_remove" =>
        [
            \Ivoz\Provider\Domain\Service\Friend\DeleteInterVpbx::class => 200,
        ],
        "error_handler" =>
        [
            \Ivoz\Provider\Domain\Service\Friend\PersistErrorHandler::class => 200,
        ],
    ];

    protected function addService(string $event, LifecycleEventHandlerInterface|DomainEventSubscriberInterface $service): void
    {
        Assertion::isInstanceOf($service, FriendLifecycleEventHandlerInterface::class);
        $this->services[$event][] = $service;
    }
}
